#ifndef _ARRAY_H_
#define _ARRAY_H_

#include <stdlib.h>
#include <string.h>
//#include <assert.h>

const int DEFAULT_CAPACITY = 3;

//---------------------------------------------------------------------------
template <typename Type, typename Compare> 
class Array
{
  public:
	explicit Array(int _capacity = DEFAULT_CAPACITY);
	explicit Array(const Array<Type, Compare>& array);
	~Array();
	inline Type*& operator[](int position);
	inline int count() const;
	inline void shrink();
	void add(Type* item);
	void insert(Type* item, int position);
	void replace(Type* item, Type* item_new, bool destroy = false);
	void remove(Type* item, bool destroy = false);
	void erase(int position, bool destroy = false);
	inline void clean(bool destroy = false);
	int locate(Type* item) const;
	int find(Type* item, int position) const;
	void sort();

  protected:
	Type** items;
	int capacity;
	int size;
	Compare compare;

	void resize(int _capacity);
	inline void expand();
};

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
Array<Type, Compare>::Array(int _capacity)
: items(new Type*[_capacity]), capacity(_capacity), size(0)
{
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
Array<Type, Compare>::Array(const Array<Type, Compare>& array)
: items(new Type*[array.capacity]), capacity(array.capacity), size(array.size)
{
	for (int i=0; i < size; i++)
		items[i] = array.items[i];
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
Array<Type, Compare>::~Array()
{
	delete [] items;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
Type*& Array<Type, Compare>::operator[](int position)
{
//	assert(position < size);
	return items[position];
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
int Array<Type, Compare>::count() const
{
	return size;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::resize(int _capacity)
{
	capacity = _capacity;
	Type** tmp = NULL;
	tmp = new Type*[capacity];
	if (items && tmp)
		memcpy(tmp, items, size*sizeof(Type*));
	delete [] items;
	items = tmp;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::expand()
{
	resize( ((capacity+1)*4)/3 );
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::shrink()
{
	resize(size);
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::add(Type* item)
{
	if (size >= capacity)
		expand();
	items[size] = item;
	size++;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::insert(Type* item, int position)
{
	if (size >= capacity)
		expand();
	if (position >= size)
		position = size;
	else
		memmove(&items[position+1], &items[position], (size-position)*sizeof(Type*));
	items[position] = item;
	size++;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::replace(Type* item, Type* item_new, bool destroy)
{
  for (int i = 0; i < size; i++)
  {
    if (items[i] == item)
    {
      if (destroy)
        delete item;
      items[i] = item_new;
    }
  }
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::remove(Type* item, bool destroy)
{
	for (int i = 0; i < size; i++)
	{
		if (items[i] == item)
		{
			if (destroy)
				delete item;
			memmove(&items[i], &items[i+1], (size-i-1)*sizeof(Type*));
			i--;
			size--;
		}
	}
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::erase(int position, bool destroy)
{
	if (position < size)
	{
		if (destroy)
			delete items[position];
		memmove(&items[position], &items[position+1], (size-position-1)*sizeof(Type*));
		size--;
	}
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array<Type, Compare>::clean(bool destroy)
{
	if (destroy)
		for (int i = 0; i < size; i++)
			delete items[i];
	size = 0;
}


//---------------------------------------------------------------------------
template <typename Type, typename Compare>
int Array <Type, Compare>::locate(Type* item) const
{
	for (int i = 0; i < size; i++)
	{
		if (items[i] == item)
			return i;
	}
	return -1;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
int Array <Type, Compare>::find(Type* item, int position) const
{
	for (int i = position; i < size; i++)
	{
		if (compare(item, items[i]) == 0)
			return i;
	}
	return -1;
}

//---------------------------------------------------------------------------
template <typename Type, typename Compare>
void Array <Type, Compare>::sort()
{
	qsort(items, size, sizeof(Type*), compare);
}

#endif
