#ifndef _HASHSET_H_
#define _HASHSET_H_

const int DEFAULT_POWER = 10;

//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
class HashSet
{
public:
	explicit HashSet(unsigned _power = DEFAULT_POWER);
	~HashSet();
	void clean(bool destroy = false);
	inline unsigned count();
	void add(Type* item);
	Type* find(Type* item) const;
	void remove(Type* item, bool destroy = false);
protected:
    struct Node
    {
      Type* item;
      Node* next;
      Node(Type* _item, Node* _next)
      {
        item = _item;
        next = _next;
      }
    };
   
    Node** nodes;
    unsigned power;
    unsigned capacity;
    unsigned size;
    Hash hash;
    Equal equal;
};


//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
HashSet<Type, Hash, Equal>::HashSet(unsigned _power)
: power(_power), capacity((unsigned)1<<_power), size(0)
{
	nodes = new Node* [capacity];
	memset(nodes, 0, sizeof(Node*)*capacity);
}
    
//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
HashSet<Type, Hash, Equal>::~HashSet()
{
	clean(false);
	delete [] nodes;
}
	
//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
void HashSet<Type, Hash, Equal>::clean(bool destroy)
{
	if (size > 0) 
	{
		for (unsigned i = 0; i < capacity; i++)
		{
			while (nodes[i])
			{
				Node* node = nodes[i];
				nodes[i] = nodes[i]->next;
				if (destroy)
					delete node->item;
				delete node;
				size--;
			}
		}
	}
}

//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
unsigned HashSet<Type, Hash, Equal>::count()
{
	return size;
}

//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
void HashSet<Type, Hash, Equal>::add(Type* item)
{
	unsigned position = hash(item) & (capacity-1);
	nodes[position] = new Node(item, nodes[position]);
	size++;
}
	
//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
Type* HashSet<Type, Hash, Equal>::find(Type* item) const
{
	if (size > 0)
	{
		unsigned position = hash(item) & (capacity-1);
		Node* node = nodes[position];
		while (node)
		{
			if ( equal(item, node->item) )
				return node->item;
			else
				node = node->next;
		}
	}
	return NULL;
}

//---------------------------------------------------------------------------
template <typename Type, typename Hash, typename Equal>
void HashSet<Type, Hash, Equal>::remove(Type* item, bool destroy)
{
	if (size > 0)
	{
	 	unsigned position = hash(item) & (capacity-1);
		Node* node = nodes[position];
		Node* node_tmp;
		while (node)
		{
			if ( equal(item, node->item) )
			{
				node_tmp = node;
				node = node->next;
				if (destroy)
					delete node_tmp->item;
				delete node_tmp;
				return;
			}
			else
				node=node->next;
		}
	}
}

#endif
