#include <stdio.h>
#include "messages.h"
#include "indie_params.h"
#include "sv_circuit_imp_opt_layer.h"

//---------------------------------------------------------------------------
int main(int argc, char* argv[])
{
  Progress("Reading command line parameters");
  if (indie_process_params(argc, argv))
  {
    Progress("Creating circuit object");
    CircuitImpOptLayer* circuit = new CircuitImpOptLayer();
    Module* module_top = NULL;
    // read predefined modules
    Progress("Reading library modules");
    circuit->readVerilog(_indie_include_file_names, MODULE_TYPE_ELEMENTARY);
    // read properties
    Progress("Reading properties");
    circuit->readProperties(_indie_properties_file_names);
    // read circuit
    Progress("Reading input files");
    circuit->readVerilog(_indie_input_file_names, MODULE_TYPE_COMPOUND);
    // calc hierarchy of the circuit
    Progress("Calculating circuit hierarchy");
    circuit->calcHierarchy();
    // find the topmost module
    Progress("Calculating the topmost module");
    module_top = circuit->calcTop();
    // sort implementations
    circuit->sortImplementations();
    // calc implementations
    Progress("Calculating implementations");
    circuit->calcImplementations();
    // calc worst-case isochronic forks
    Progress("Calculating worst-case number of timing assumptions");
    circuit->calcAllForks("Worst-case number of timing assumptions");
    // calc full completion detection
    Progress("Calculating completion detection");
    circuit->calcCompletion("Completion logic", true);
    // calc min completion detection
    Progress("Calculating min completion detection");
    circuit->calcCompletion("Min completion", false);
    // calc tables
    Progress("Calculating tables");
    circuit->calcTables(_indie_implementation_names);
    // optimise tables
    Progress("Optimising tables");
    circuit->optimiseTables();
    // optimise implementations
    Progress("Calculating optimised implementation");
    circuit->setMaxIteration(_indie_max_iteration);
    if (_indie_greedy_layerwise)
      circuit->calcLayerwiseImplementation();
    else
      circuit->calcOptImplementation();
    // calc optimised isochronic forks
    Progress("Calculating optimised number of timing assumptions");
    circuit->calcOptForks("Optimised number of timing assumptions");
    // calc implementation
    Progress("Calculation orphans");
    circuit->calcOrphans("Orphans in optimised implementation");
    // write the resultunt circuit into the output file
    Progress("Writing output file");
    circuit->writeVerilogImp(_indie_output_file_name, MODULE_TYPE_COMPOUND, _indie_statistics_level);
    // clean up
    Progress("Cleaning up circuit object");
    delete circuit;
  }
  Progress("Cleaning up command line parameters");
  indie_clear_params();
  Progress("Done");
  return 1;
}
