#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "param.h"
#include "str_utils.h"
#include "indie_msgs.h"
#include "indie_params.h"

char* _indie_program_name;
char* _indie_input_file_names;
char* _indie_output_file_name;
char* _indie_include_file_names;
char* _indie_properties_file_names;
char* _indie_implementation_names;
int _indie_statistics_level;
int _indie_max_iteration;
bool _indie_greedy_layerwise;

//---------------------------------------------------------------------------
bool indie_process_params(int argc, char** argv)
{
  int i;
  int stdout_num = 1;
  bool result = true;

  indie_clear_params();
  char* program_name = strrchr(argv[0], '/');
  if (program_name == NULL)
    _indie_program_name = namecopy(argv[0]);
  else
    _indie_program_name = namecopy(&program_name[1]);

  if (argc < 2)
    i = 0;
  else
    i = 1;

  while (i > 0)
  {
    // [_indie_input_file_names]
    if (argv[i][0] != '-')
    {
      if (_indie_input_file_names == NULL)
        _indie_input_file_names = namecopy(argv[i]);
      else
      {
        char* input_file_name = name2pattern("%s,%s", _indie_input_file_names, argv[i]);
        delete [] _indie_input_file_names;
        _indie_input_file_names = input_file_name;
      }
      i++;
      if (i >= argc) i = 0;
      continue;
    }
    else
    // -o [_indie_output_file_name] <=> --output [_indie_output_file_name]
    if (param_string(argc, argv, &i, "o", &_indie_output_file_name) ||
        param_string(argc, argv, &i, "-output", &_indie_output_file_name))
    {
    }
    else
    // -i [_indie_include_file_names] <=> --include [_indie_include_file_names]
    if (param_string(argc, argv, &i, "i", &_indie_include_file_names) ||
        param_string(argc, argv, &i, "-include", &_indie_include_file_names))
    {
    }
    else
    // -p [_indie_properties_file_names] <=> --properties [_indie_properties_file_names]
    if (param_string(argc, argv, &i, "p", &_indie_properties_file_names) ||
        param_string(argc, argv, &i, "-properties", &_indie_properties_file_names))
    {
    }
    else
    // -os [_indie_implementation_names] <=> --optimisation-space [_indie_implementation_names]
    if (param_string(argc, argv, &i, "os", &_indie_implementation_names) ||
        param_string(argc, argv, &i, "-optimisation-space", &_indie_implementation_names))
    {
    }
    else
    // -gl <=> --greedy-layerwise
    if (param_bool(argc, argv, &i, "gl") ||
        param_bool(argc, argv, &i, "-greedy-layerwise"))
    {
      _indie_greedy_layerwise = true;
    }
    else
    // -sl [_indie_statistics_level] <=> --statistics-level [_indie_statistics_level]
    if (param_integer(argc, argv, &i, "sl", &_indie_statistics_level) ||
        param_integer(argc, argv, &i, "-statistics-level", &_indie_statistics_level))
    {
    }
    else
    // -mi [_indie_max_iteration] <=> --max-iteration [_indie_max_iteration]
    if (param_integer(argc, argv, &i, "mi", &_indie_max_iteration) ||
        param_integer(argc, argv, &i, "-max-iteration", &_indie_max_iteration))
    {
    }
    else
    // -h <=> --help
    if (param_bool(argc, argv, &i, "h") ||
        param_bool(argc, argv, &i, "-help"))
    {
      if (i != 0)
        Warning("<%s> must be the only flag",argv[i-1]);
      char* _indie_help = indie_get_help(_indie_program_name);
      fprintf(stderr, _indie_help);
      delete [] _indie_help;
      result = false;
    }
    else
    // -v <=> --version
    if (param_bool(argc, argv, &i, "v") ||
        param_bool(argc, argv, &i, "-vesion"))
    {
      if (i != 0)
        Warning("\"%s\" must be the only flag",argv[i-1]);
      char* _indie_version = indie_get_version(_indie_program_name);
      fprintf(stderr, _indie_version);
      delete [] _indie_version;
      result = false;
    }
    else
    // UNRECOGNIZED FLAG
    {
      Error("unrecognized flag \"%s\"", argv[i]);
      result = false;
    }
  }
  indie_correct_params();
  return result;
}

//---------------------------------------------------------------------------
void indie_clear_params()
{
  delete [] _indie_program_name; _indie_program_name = NULL;
  delete [] _indie_input_file_names; _indie_input_file_names = NULL;
  delete [] _indie_output_file_name; _indie_output_file_name = NULL;
  delete [] _indie_include_file_names; _indie_include_file_names = NULL;
  delete [] _indie_properties_file_names; _indie_properties_file_names = NULL;
  delete [] _indie_implementation_names; _indie_implementation_names = NULL;
  _indie_greedy_layerwise = false;
  _indie_statistics_level = -1;
  _indie_max_iteration = -1;
}

//---------------------------------------------------------------------------
void indie_correct_params()
{
  if (_indie_program_name == NULL)
    _indie_program_name = namecopy(_INDIE_PROGRAM_NAME);
  if (_indie_input_file_names == NULL)
    _indie_input_file_names = namecopy(_INDIE_INPUT_FILE_NAMES);
  if (_indie_output_file_name == NULL)
    _indie_output_file_name = namecopy(_INDIE_OUTPUT_FILE_NAME);
  if (_indie_include_file_names == NULL)
    _indie_include_file_names = namecopy(_INDIE_INCLUDE_FILE_NAMES);
  if (_indie_properties_file_names == NULL)
    _indie_properties_file_names = namecopy(_INDIE_PROPERTIES_FILE_NAMES);
  // if no input file specified read it from standard input
  if (_indie_input_file_names == NULL)
    _indie_input_file_names = namecopy(NAME_STDIO);
  // if no output file specified write it to standard output
  if (_indie_output_file_name == NULL)
    _indie_output_file_name = namecopy(NAME_STDIO);
  if (_indie_implementation_names == NULL)
    _indie_implementation_names = _INDIE_IMPLEMENTATION_NAMES;
  if (_indie_statistics_level < 0)
    _indie_statistics_level = _INDIE_STATISTICS_LEVEL;
  if (_indie_max_iteration < 0)
    _indie_max_iteration = _INDIE_MAX_ITERATION;

  // count the number of files read from standard input and written to standard output
  int stdin_num = 0;
  int stdout_num = 0;

  stdin_num += namecount(_indie_input_file_names, NAME_STDIO);
  stdin_num += namecount(_indie_include_file_names, NAME_STDIO);
  stdin_num += namecount(_indie_properties_file_names, NAME_STDIO);
  stdout_num += namecount(_indie_output_file_name, NAME_STDIO);

  if (stdin_num > 1)
    Warning("%d files will be read from standard input", stdin_num);
  if (stdout_num > 1)
    Warning("%d files will be written to standard output", stdout_num);
}
