%{
#include "messages.h"
#include "str_utils.h"
#include "sv_property.h"
#include "properties_y.h"

//#define DEBUG_LEX 1
#ifdef DEBUG_LEX
  #include <stdio.h>
  #define DEBUG_LEX_MESSAGE(msg) fprintf(stderr, msg)
#else
  #define DEBUG_LEX_MESSAGE(msg) // msg
#endif

%}

%option yylineno

WS                 [\n\r\t\b ]
COMMENT_C_OPEN     "/*"
COMMENT_C_CLOSE    "*/"
COMMENT_CPP_OPEN   "//"
COMMENT_CPP_CLOSE  \n
DIGIT              [0-9]
ALPHA              [a-zA-Z_]
ALPHANUM           [a-zA-Z_0-9]
ID                 {ALPHA}+({DIGIT}*{ALPHA}*)*
INUMBER            {DIGIT}{DIGIT}*
RNUMBER1           {DIGIT}*\.{DIGIT}+
RNUMBER2           {DIGIT}+\.{DIGIT}*
RNUMBER            {RNUMBER1}|{RNUMBER2}
UNDEFINED          "-1"

%x COMMENT_C
%x COMMENT_CPP

%%
%{
int comment_depth = 0;
%}

{WS}+              {
  /* eat white spaces */
}

{COMMENT_C_OPEN}   {
  DEBUG_LEX_MESSAGE("COMMENT_C_OPEN\n");
  comment_depth = 1;
  BEGIN COMMENT_C;
}

<COMMENT_C>{COMMENT_C_OPEN} {
  DEBUG_LEX_MESSAGE("COMMENT_C_OPEN\n");
  comment_depth++;
}

<COMMENT_C>{COMMENT_C_CLOSE} {
  DEBUG_LEX_MESSAGE("COMMENT_C_CLOSE\n");
  comment_depth--;
  if (comment_depth == 0)
    BEGIN INITIAL;
}

<COMMENT_C>.|\n    {
  /* eat comments */
}

{COMMENT_CPP_OPEN} {
  DEBUG_LEX_MESSAGE("COMMENT_CPP_OPEN\n");
  BEGIN COMMENT_CPP;
}

<COMMENT_CPP>{COMMENT_CPP_CLOSE} {
  DEBUG_LEX_MESSAGE("COMMENT_CPP_CLOSE\n");
  BEGIN INITIAL;
}

<COMMENT_CPP>.     {
  /* eat comments */
}

{ID}               {
  DEBUG_LEX_MESSAGE("ID: "); DEBUG_LEX_MESSAGE(yytext); DEBUG_LEX_MESSAGE("\n");
  properties_lval.string = namecopy(yytext);
  return YYID;
}

{INUMBER}          {
  DEBUG_LEX_MESSAGE("INUMBER: "); DEBUG_LEX_MESSAGE(yytext); DEBUG_LEX_MESSAGE("\n");
  properties_lval.inumber = atoi(yytext);
  return YYINUMBER;
}

{UNDEFINED}        {
  DEBUG_LEX_MESSAGE("UNDEFINED: "); DEBUG_LEX_MESSAGE(yytext); DEBUG_LEX_MESSAGE("\n");
  properties_lval.inumber = -1;
  return YYUNDEFINED;
}

.                  {
  DEBUG_LEX_MESSAGE(yytext); DEBUG_LEX_MESSAGE("\n");
  return yytext[0];
}

<<EOF>>            {
  DEBUG_LEX_MESSAGE("<EOF>\n");
  yyterminate();
  return 0;
}

%%

int yywrap()
{
  properties_lineno = 0;
  // Free the buffer which remains un-freed due to a flex bug
  yy_delete_buffer(yy_current_buffer);
  yy_init = 1;
  return 1;
}
