#ifndef _SV_CIRCUIT_H_
#define _SV_CIRCUIT_H_
#include <stdio.h>
#include "sv_object.h"

class Circuit;

#include "sv_module.h"

//---------------------------------------------------------------------------
class Circuit : public Object
{
  public:
    Circuit(const char* _name = NULL, ModuleArray* _modules = NULL, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Circuit(const Circuit &circuit);
    ~Circuit();

    void addModule(Module* module);
    void addModules(ModuleArray* _modules);
    void removeModule(Module* module);
    Module* findModule(const char* module_name) const;

    Module* setTop(const char* module_name);
    Module* getTop() const;
    Module* calcTop();

    void calcHierarchy();
    bool isHierarchical() const;

    void readVerilog(const char* file_names, ModuleType module_type = MODULE_TYPE_COMPOUND);
    void writeVerilog(const char* file_name, ModuleType module_type = MODULE_TYPE_COMPOUND) const;

  protected:
    ModuleArray* modules;
    ModuleHashSet* module_hash_set;
    Module* top;

    void clean();
    void sortConnections();
    void calcConnections();
    void findHierarchy(Module* module, int depth);
    int getDepth() const;
    void writeVerilogModuleInterface(FILE* file, Module* module) const;
    void writeVerilogModulePorts(FILE* file, Module* module) const;
    void writeVerilogModuleParameters(FILE* file, Module* module) const;
    void writeVerilogModuleInstances(FILE* file, Module* module) const;
    void writeVerilogModule(FILE* file, Module* module) const;
};

#endif

