#ifndef _SV_CIRCUIT_IMP_H_
#define _SV_CIRCUIT_IMP_H_

#include <stdio.h>
#include "sv_implementation.h"
#include "sv_property.h"
#include "sv_circuit.h"

//---------------------------------------------------------------------------
class CircuitImp : public Circuit
{
  public:
    ImplementationArray* implementations;

    CircuitImp(const char* _name = NULL, ModuleArray* _modules = NULL, ImplementationArray* _implementations = NULL,
      PropertyArray* _properies = NULL, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    CircuitImp(const CircuitImp& circuit_imp);
    ~CircuitImp();

    void addImplementation(Implementation* implementation, int position = -1);
    void addImplementations(ImplementationArray* _implementations);
    void removeImplementation(Implementation* implementation);
    Implementation* findImplementation(const char* implementation_name) const;

    void addProperty(Property* property, int position = -1);
    void addProperties(PropertyArray* _properties);
    void removeProperty(Property* property);
    Property* initProperty(const char* property_name, int value = -1);
    Property* findProperty(const char* property_name) const;

    void sortImplementations();
    void calcImplementations();

    void calcAllForks(const char* forks_name);
    void calcCompletion(const char* completion_name, bool full);
    
    void readProperties(const char* file_names);
    void writeProperties(const char* file_name) const;
    void writeImplementations(FILE* file) const;
    void writeModuleImplementations(FILE* file, Module* module) const;
    void writeVerilogImp(const char* file_name, ModuleType module_type = MODULE_TYPE_COMPOUND, int statistics_level = 0) const;

  protected:
    PropertyArray* properties;
    PropertyHashSet* property_hash_set;
    
    void clean();
    void calcModuleImplementations(Module* module);
    int calcModuleAllForks(Module* module, const char* forks_name);
    int calcModuleCompletion(Module* module, const char* completion_name, bool full);
    void findElementC_x2x3(int input_num, int &x2, int &x3);
    int calcElementC(int input_num);
};

#endif
