#ifndef _SV_CIRCUIT_IMP_OPT_H_
#define _SV_CIRCUIT_IMP_OPT_H_

#include "sv_set.h"
#include "sv_region.h"
#include "sv_circuit_imp.h"

//---------------------------------------------------------------------------
class CircuitImpOpt : public CircuitImp
{
  public:
    CircuitImpOpt(const char* _name = NULL, ModuleArray* _modules = NULL, ImplementationArray* _implementations = NULL,
      PropertyArray* _properies = NULL, unsigned _iteration_max = 1000000, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    ~CircuitImpOpt();
    

    void setMaxIteration(unsigned _iteration_max);
    unsigned getMaxIteration();
    void setOptimisedImplementationName(const char* _optimised_implementation_name);
    const char* getOptimisedImplementationName() const;

    void calcTables(const char* implementation_names);
    void optimiseTables();
    void calcOptImplementation();
    void calcOptForks(const char* forks_name);
    void calcOrphans(const char* orphans_name);

  protected:
    char* optimised_implementation_name;
    unsigned iteration_max;
    int indent;
    bool region_optimisation;
    
    int calcVariationCode(const char* variation_name, Module* module) const;
    int calcSolutionValue(Region* region);
    bool findRegionImplementation(Region* region, int start);
    int calcRegionImplementation(Region* region);
    void findRegionInstance(Set* set, Region* region, Instance* instance);
    void findRegionNet(Set* set, Region* region, Net* net);
    void calcRegionData(Region* region);
    void saveRegionData(Region* region);
    void cleanRegionData(Region* region);
    int calcRegionwiseImplementation(Set* set);
    int calcModuleOptImplementation(Module* module);
    int calcModuleOptForks(Module* module, const char* forks_name);
    int calcModuleOrphans(Module* module, const char* orphans_name);
};

//---------------------------------------------------------------------------
inline void CircuitImpOpt::setOptimisedImplementationName(const char* _optimised_implementation_name)
{
  delete [] optimised_implementation_name;
  optimised_implementation_name = namecopy(_optimised_implementation_name);
}

//---------------------------------------------------------------------------
inline const char* CircuitImpOpt::getOptimisedImplementationName() const
{
  return optimised_implementation_name;
}

//---------------------------------------------------------------------------
inline void CircuitImpOpt::setMaxIteration(unsigned _iteration_max)
{
  iteration_max = _iteration_max;
}

//---------------------------------------------------------------------------
inline unsigned CircuitImpOpt::getMaxIteration()
{
  return iteration_max;
}

#endif
