#include "messages.h"
#include "sverilog.h"
#include "sv_object.h"
#include "sv_region.h"
#include "sv_indicator.h"
#include "sv_circuit_imp_opt_layer.h"
#include "properties.h"

//---------------------------------------------------------------------------
CircuitImpOptLayer::CircuitImpOptLayer(const char* _name, ModuleArray* _modules, ImplementationArray* _implementations,
PropertyArray* _properties, unsigned _iteration_max, Tag _tag, void* _data)
: CircuitImpOpt(_name, _modules, _implementations, _properties, _iteration_max, _tag, _data)
{
}

//---------------------------------------------------------------------------
CircuitImpOptLayer::~CircuitImpOptLayer()
{
}

//---------------------------------------------------------------------------
int CircuitImpOptLayer::calcModuleLayerImplementation(Module* module, int layer_index)
{
  int result = 0;
  indent += 2;
  // build layer of instances
  char* layer_name = name2pattern("%s-%.2d", module->getName(), layer_index+1);
  Set* layer = new Set(layer_name);
  delete [] layer_name;

  Text(indent, "processing layer <%s>\n", layer->getName());
  for (int instance_index = 0; instance_index < module->instances->count(); instance_index++)
  {
    Instance* instance_cur = (*module->instances)[instance_index];
    if (instance_cur->getLayer() == layer_index)
    {
      Module* module_prot = findModule(instance_cur->getModuleName());
      if (module_prot->getType() == MODULE_TYPE_COMPOUND)
        result += calcLayerwiseModuleImplementation(module_prot);
      else
        layer->addInstance(instance_cur);
    }
  }
  // build layer of nets
  for (int port_index = 0; port_index < module->ports->count(); port_index++)
  {
    Port* port_cur = (*module->ports)[port_index];
    if ( !port_cur->testTag(TAG_HIDDEN) )
    for (int net_index = 0; net_index < port_cur->nets->count(); net_index++)
    {
      Net* net_cur = (*port_cur->nets)[net_index];
      if (net_cur->getLayer() == layer_index + 1)
      {
        bool net_is_indicated = false;
        for(int postset_index = 0; postset_index < net_cur->postset->count(); postset_index++)
        {
          Instance* instance_cur = (*net_cur->postset)[postset_index];
          Module* module_prot = findModule(instance_cur->getModuleName());
          if (module_prot->getType() == MODULE_TYPE_COMPOUND)
          {
            net_is_indicated = true;
            break;
          }                
        }
        if ( !net_is_indicated && (net_cur->postset->count() > 0) )
          layer->addNet(net_cur);
      }
    }
  }
  result += calcRegionwiseImplementation(layer);
  delete layer;
  indent -= 2;
  return result;
}

//---------------------------------------------------------------------------
int CircuitImpOptLayer::calcLayerwiseModuleImplementation(Module* module)
{
  int result = 0;
  if(module->getType() == MODULE_TYPE_COMPOUND)
  {
    indent += 2;
    if (module->testTag(TAG_PROCESSED))
    {
      Property* property_cur = findProperty(module->getName());
      if (property_cur)
      {
        Implementation* implementation_opt = property_cur->findImplementation(optimised_implementation_name);
        if (implementation_opt)
          result = implementation_opt->getValue();
      }
    }
    else
    {
      Text(indent, "processing module <%s>\n", module->getName());
      module->addTag(TAG_PROCESSED);
      int layer_count = 0;
      for (int instance_index = 0; instance_index < module->instances->count(); instance_index++)
      {
        Instance* instance_cur = (*module->instances)[instance_index]; 
        if (instance_cur->getLayer() > layer_count)
          layer_count = instance_cur->getLayer();
      }
      for(int layer_index = 0; layer_index <= layer_count; layer_index++)
        result += calcModuleLayerImplementation(module, layer_index);
      Property* property_cur = findProperty(module->getName());
      Implementation* implementation_opt = new Implementation(optimised_implementation_name, IMPLEMENTATION_TYPE_OPT, result);
      property_cur->addImplementation(implementation_opt);
    }
    indent -= 2;
  }
  return result;
}

//---------------------------------------------------------------------------
void CircuitImpOptLayer::calcLayerwiseImplementation()
{
  if ( !top ) calcTop();
  if ( !top )
    Warning("cannot calc implementations because the top module is not defined");
  else
  {
    for (int module_index = 0; module_index < modules->count(); module_index++)
      (*modules)[module_index]->removeTag(TAG_PROCESSED);
    indent = 0;
    int value_opt = calcLayerwiseModuleImplementation(top);
    Implementation* implementation_opt = new Implementation(optimised_implementation_name, IMPLEMENTATION_TYPE_OPT, value_opt);
    addImplementation(implementation_opt);
  }
}
