#ifndef _SV_CONNECTION_H_
#define _SV_CONNECTION_H_

#include "array.h"
#include "hashset.h"
#include "sv_object.h"

class Connection;
typedef Array <Connection, ObjectCompareName> ConnectionArray;
typedef HashSet <Connection, ObjectHashName, ObjectEqualName> ConnectionHashSet;

enum ConnectionType
{
  CONNECTION_TYPE_UNDEFINED,
  CONNECTION_TYPE_INPUT,
  CONNECTION_TYPE_OUTPUT,
  CONNECTION_TYPE_INOUT
};

#include "sv_net.h"
#include "sv_instance.h"

//---------------------------------------------------------------------------
class Connection : public Object
{
  public:

    NetArray* nets;
    
    Connection(const char* _name = NULL, ConnectionType _type = CONNECTION_TYPE_UNDEFINED, NetArray* _nets = NULL);
    Connection(const Connection& connection);
    ~Connection();
	
    void setType(ConnectionType _type);
    ConnectionType getType() const;
    void setContainer(Instance* _container);
    Instance* getContainer() const;

    void addNet(Net* net, int position = -1);
    void addNets(NetArray* nets);

  protected:
    ConnectionType type;
    Instance* container;
};

//---------------------------------------------------------------------------
inline void Connection::setType(ConnectionType _type)
{
  type = _type;
}

//---------------------------------------------------------------------------
inline ConnectionType Connection::getType() const
{
  return type;
}

//---------------------------------------------------------------------------
inline void Connection::setContainer(Instance* _container)
{
  container = _container;
}

//---------------------------------------------------------------------------
inline Instance* Connection::getContainer() const
{
  return container;
}

#endif

