#ifndef _SV_IMPLEMENTATION_H_
#define _SV_IMPLEMENTATION_H_

#include "array.h"
#include "hashset.h"
#include "sv_object.h"

class Implementation;
typedef Array <Implementation, ObjectCompareName> ImplementationArray;
typedef HashSet <Implementation, ObjectHashName, ObjectEqualName> ImplementationHashSet;

//---------------------------------------------------------------------------
enum ImplementationType
{
  IMPLEMENTATION_TYPE_UNDEFINED,
  IMPLEMENTATION_TYPE_NONE,
  IMPLEMENTATION_TYPE_PARTIAL,
  IMPLEMENTATION_TYPE_FULL,
  IMPLEMENTATION_TYPE_OPT,
  IMPLEMENTATION_TYPE_FORKS,
  IMPLEMENTATION_TYPE_COMPLETION,
  IMPLEMENTATION_TYPE_ORPHANS
};

//---------------------------------------------------------------------------
class Implementation : public Object
{
  public:
    Implementation(const char* _name, ImplementationType _type = IMPLEMENTATION_TYPE_UNDEFINED, 
      int _value = -1, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Implementation(const Implementation& implementation);
	~Implementation();

    void setValue(int _value);
    int getValue() const;
    void setType(ImplementationType _type);
    ImplementationType getType() const;

  private:
    int value;
    ImplementationType type;
};

//---------------------------------------------------------------------------
inline void Implementation::setValue(int _value)
{
  value = _value;
}

//---------------------------------------------------------------------------
inline int Implementation::getValue() const
{
  return value;
}

//---------------------------------------------------------------------------
inline void Implementation::setType(ImplementationType _type)
{
  type = _type;
}

//---------------------------------------------------------------------------
inline ImplementationType Implementation::getType() const
{
  return type;
}

#endif
