#include "sv_instance.h"
#include "str_utils.h"
#include "messages.h"

//---------------------------------------------------------------------------
Instance::Instance(const char* _name, int _index, const char* _module_name, ConnectionArray* _connections) 
: ObjectExt(_name, _index), module_name(NULL), container(NULL)
{
  setModuleName(_module_name);
  if (_connections)
    connections = new ConnectionArray(_connections->count());
  else
    connections = new ConnectionArray(1);
  addConnections(_connections); 
}

//---------------------------------------------------------------------------
Instance::Instance(const Instance& instance) 
: ObjectExt(instance), module_name(NULL), container(instance.container)
{
  setModuleName(instance.module_name);
  connections = new ConnectionArray(instance.connections->count());
  for (int i = 0; i < instance.connections->count(); i++)
    addConnection( new Connection(*(*instance.connections)[i]) );
}

//---------------------------------------------------------------------------
Instance::~Instance()
{
  clean();
  delete connections; connections = NULL;
  delete [] module_name; module_name = NULL;
}

//---------------------------------------------------------------------------
void Instance::clean()
{
  connections->clean(true);
}

//---------------------------------------------------------------------------
void Instance::addConnection(Connection* connection, int position)
{
  connection->setContainer(this);
  if (position < 0)
    connections->add(connection);
  else
    connections->insert(connection, position);
}

//---------------------------------------------------------------------------
void Instance::addConnections(ConnectionArray* _connections)
{
  if (_connections)
  for (int connection_index = 0; connection_index < _connections->count(); connection_index++)
    addConnection((*_connections)[connection_index]);
}

//---------------------------------------------------------------------------
void Instance::removeConnection(Connection* connection)
{
  connections->remove(connection, true);
}

//---------------------------------------------------------------------------
Connection* Instance::findConnection(const char* connection_name) const
{
  Connection* connection_mask = new Connection(connection_name);
  int connection_index = connections->find(connection_mask, 0);
  delete connection_mask;
  if (connection_index >= 0)
    return (*connections)[connection_index];
  else
    return NULL;
}
