#ifndef _SV_INSTANCE_H_
#define _SV_INSTANCE_H_

#include "array.h"
#include "hashset.h"
#include "sv_object.h"

class Instance;
typedef Array <Instance, ObjectCompareName> InstanceArray;
typedef HashSet <Instance, ObjectHashName, ObjectEqualName> InstanceHashSet;

#include "sv_object_ext.h"
#include "sv_net.h"
#include "sv_connection.h"
#include "sv_module.h"

//---------------------------------------------------------------------------
class Instance : public ObjectExt
{
  public:
    ConnectionArray* connections;
  
    Instance(const char* _name, int _index = -1, const char* _module_name = NULL, ConnectionArray* _connections = NULL);
    Instance(const Instance& instance);
    ~Instance();
	
    void setModuleName(const char* _module_name);
    const char* getModuleName() const;
    void setContainer(Module* _container);
    Module* getContainer() const;

    void addConnection(Connection* connection, int position = -1);
    void addConnections(ConnectionArray* _connections);
    void removeConnection(Connection* connection);
    Connection* findConnection(const char* connection_name) const;

  private:
    char* module_name;
    Module* container;

    void clean();
};

//---------------------------------------------------------------------------
inline void Instance::setModuleName(const char* _module_name)
{
  delete [] module_name;
  module_name = namecopy(_module_name);
}

//---------------------------------------------------------------------------
inline const char* Instance::getModuleName() const
{
  return module_name;
}

//---------------------------------------------------------------------------
inline void Instance::setContainer(Module* _container)
{
  container = _container;
}

//---------------------------------------------------------------------------
inline Module* Instance::getContainer() const
{
  return container;
}

#endif
