#ifndef _SV_NET_H_
#define _SV_NET_H_

#include "array.h"
#include "hashset.h"
#include "sv_object.h"

class Net;
typedef Array <Net, ObjectCompareName> NetArray;
typedef HashSet <Net, ObjectHashName, ObjectEqualName> NetHashSet;

extern "C" const char* NET_NAME_GND;
extern "C" const char* NET_NAME_VDD;

#include "sv_object_ext.h"
#include "sv_delay.h"
#include "sv_port.h"
#include "sv_instance.h"

//---------------------------------------------------------------------------
class Net : public ObjectExt
{
  public:
    InstanceArray* preset;
    InstanceArray* postset;

    Net(const char* _name = NULL, int _index = -1, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Net(const Net& _net);
    ~Net();

    void setContainer(Port* container);
    Port* getContainer() const;
    void setDepth(int _depth);
    int getDepth() const;
    void setLayer(int _layer);
    int getLayer() const;

    void addPreset(Instance* instance);
    void removePreset(Instance* instance);
    void cleanPreset();
    void addPostset(Instance* instance);
    void removePostset(Instance* instance);
    void cleanPostset();

  protected:
    int depth;
    int layer;
    Port* container;
};

//---------------------------------------------------------------------------
inline void Net::setContainer(Port* _container)
{
  container = _container;
}

//---------------------------------------------------------------------------
inline Port* Net::getContainer() const
{
  return container;
}

//---------------------------------------------------------------------------
inline void Net::setDepth(int _depth)
{
  depth = _depth;
}

//---------------------------------------------------------------------------
inline int Net::getDepth() const
{
  return depth;
}

//---------------------------------------------------------------------------
inline void Net::setLayer(int _layer)
{
  layer = _layer;
}

//---------------------------------------------------------------------------
inline int Net::getLayer() const
{
  return layer;
}

//---------------------------------------------------------------------------
inline void Net::addPreset(Instance* instance)
{
  if (preset->locate(instance) < 0)
    preset->add(instance);
}

//---------------------------------------------------------------------------
inline void Net::removePreset(Instance* instance)
{
  preset->remove(instance, false);
}

//---------------------------------------------------------------------------
inline void Net::cleanPreset()
{
  preset->clean(false);
}

//---------------------------------------------------------------------------
inline void Net::addPostset(Instance* instance)
{
  if (postset->locate(instance) < 0)
    postset->add(instance);
}

//---------------------------------------------------------------------------
inline void Net::removePostset(Instance* instance)
{
  postset->remove(instance, false);
}

//---------------------------------------------------------------------------
inline void Net::cleanPostset()
{
  postset->clean(false);
}

#endif
