#include "str_utils.h"
#include "messages.h"
#include "sv_object_ext.h"
#include "sv_port.h"
#include "sv_range.h"
#include "sv_delay.h"
#include "sv_net.h"

//---------------------------------------------------------------------------
Port::Port(const char* _name, PortType _type, Range* _range, Delay* _delay, Tag _tag, void* _data) 
: Object(_name, _tag, _data), type(_type), range(NULL), delay(NULL), nets(NULL)
{
  setRange(_range);
  setDelay(_delay);
}

//---------------------------------------------------------------------------
Port::Port(const Port& port) 
: Object(port), range(NULL), delay(NULL), nets(NULL)
{
  setRange(port.range);
  setDelay(port.delay);
}

//---------------------------------------------------------------------------
Port::~Port()
{
  clean();
  delete nets; nets = NULL;
  delete range; range = NULL;
}

//---------------------------------------------------------------------------
void Port::clean()
{
  nets->clean(true);
}

//---------------------------------------------------------------------------
void Port::setRange(Range* _range)
{
  delete range; range = NULL;
  if (nets)
    nets->clean(true);
  if (_range)
    range = new Range(*_range);
  if (range)
  {
    nets = new NetArray(range->getWidth());
    for (int net_index = range->getFrom(); net_index != range->getTo() + range->getInc(); net_index += range->getInc())
    {
      Net* net = new Net(name, net_index, tag);
      addNet(net);
    }
  }
  else
  {
    nets = new NetArray(1);
    Net* net = new Net(name, -1, tag);
    addNet(net);
  }
}

//---------------------------------------------------------------------------
void Port::setDelay(Delay* _delay)
{
  delete delay; delay = NULL;
  if (_delay)
    delay = new Delay(*_delay);
}

//---------------------------------------------------------------------------
void Port::addNet(Net* net, int position)
{
  net->setContainer(this);
  if (position < 0)
    this->nets->add(net);
  else
    this->nets->insert(net, position);
}

//---------------------------------------------------------------------------
Net* Port::findNet(int index) const
{
  Net* net = NULL;
  int position = 0;
  if (range != NULL)
      position = range->getPosition(index);
  if ( (position >= 0) && (nets->count() > position) )
    net = (*nets)[position];
  return net;
}
