#ifndef _SV_PORT_H_
#define _SV_PORT_H_

#include "array.h"
#include "hashset.h"
#include "sv_object.h"

class Port;
typedef Array <Port, ObjectCompareName> PortArray;
typedef HashSet <Port, ObjectHashName, ObjectEqualName> PortHashSet;

enum PortType 
{ 
  PORT_TYPE_UNDEFINED,  
  PORT_TYPE_INPUT, 
  PORT_TYPE_OUTPUT, 
  PORT_TYPE_INOUT, 
  PORT_TYPE_WIRE,
  PORT_TYPE_REG
};

#include "sv_object_ext.h"
#include "sv_range.h"
#include "sv_delay.h"
#include "sv_net.h"
#include "sv_module.h"

//---------------------------------------------------------------------------
class Port : public Object
{
  public:
    NetArray* nets;

    Port(const char* _name, PortType _type = PORT_TYPE_UNDEFINED, Range* _range = NULL, 
      Delay* _delay = NULL, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Port(const Port& port);
    ~Port();
	
    void setType(PortType _type);
    PortType getType() const;
    void setRange(Range* _range);
    Range* getRange() const;
    void setDelay(Delay* _delay);
    Delay* getDelay() const;
    void setContainer(Module* _container);
    Module* getContainer() const;

    void addNet(Net* net, int position = -1);
    Net* findNet(int index) const;

  protected:
    PortType type;
    Range* range;
    Delay* delay;
    Module* container;

    void clean();
};

//---------------------------------------------------------------------------
inline void Port::setType(PortType _type)
{
  type = _type;
}

//---------------------------------------------------------------------------
inline PortType Port::getType() const
{
  return type;
}

//---------------------------------------------------------------------------
inline Range* Port::getRange() const
{
  return range;
}

//---------------------------------------------------------------------------
inline Delay* Port::getDelay() const
{
  return delay;
}

//---------------------------------------------------------------------------
inline void Port::setContainer(Module* _container)
{
  container = _container;
}

//---------------------------------------------------------------------------
inline Module* Port::getContainer() const
{
  return container;
}

#endif

