#include "sv_object.h"
#include "sv_property.h"

//---------------------------------------------------------------------------
Property::Property(const char* _name, ImplementationArray* _implementations, Tag _tag, void* _data)
:Object(_name, _tag, _data)
{
  if (_implementations)
    implementations = new ImplementationArray(_implementations->count());
  else
    implementations = new ImplementationArray(0);
  addImplementations(_implementations);
}

//---------------------------------------------------------------------------
Property::Property(Property& property)
:Object(property)
{
  implementations = new ImplementationArray(property.implementations->count());
  for (int implementation_index = 0; implementation_index < property.implementations->count(); implementation_index++)
  {
    Implementation* implementation_cur = (*property.implementations)[implementation_index];
    addImplementation( new Implementation(*implementation_cur) );
  }
}

//---------------------------------------------------------------------------
Property::~Property()
{
  clean();
  delete implementations; implementations = NULL;
}

//---------------------------------------------------------------------------
void Property::clean()
{
  implementations->clean(true);
}

//---------------------------------------------------------------------------
void Property::addImplementation(Implementation* implementation, int position)
{
  if (position < 0)
    implementations->add(implementation);
  else
    implementations->insert(implementation, position);
}

//---------------------------------------------------------------------------
void Property::addImplementations(ImplementationArray* _implementations)
{
  if (_implementations)
  for (int implementation_index = 0; implementation_index < _implementations->count(); implementation_index++)
  {
    Implementation* implementation_cur = (*_implementations)[implementation_index];
    addImplementation(implementation_cur);
  }
}

//---------------------------------------------------------------------------
void Property::removeImplementation(Implementation* implementation)
{
  implementations->remove(implementation, true);
}

//---------------------------------------------------------------------------
Implementation* Property::findImplementation(const char* implementation_name) const
{
  Implementation* implementation_mask = new Implementation(implementation_name);
  int implementation_index = implementations->find(implementation_mask, 0);
  delete implementation_mask;
  if (implementation_index >= 0)
    return  (*implementations)[implementation_index];
  else
    return NULL;
}
