#ifndef _SV_REGION_H_
#define _SV_REGION_H_

#include <string.h>
#include "array.h"
#include "hashset.h"
#include "sv_object.h"
#include "sv_instance.h"
#include "sv_net.h"
#include "sv_set.h"

//---------------------------------------------------------------------------
class Region;
typedef Array<Region, ObjectCompareName> RegionArray;
typedef HashSet<Region, ObjectHashName, ObjectEqualName> RegionHashSet;

typedef unsigned* Solution;

//---------------------------------------------------------------------------
class Region : public Set
{
  public:
    Region(const char* _name, InstanceArray* _instances = NULL, NetArray* _nets = NULL, 
      unsigned _iteratio_max = 1000000, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    ~Region();

    void initSolution();
    void addSolution(int instance_index, int input_index);
    void removeSolution(int instance_index, int input_index);
    void saveSolution();
    void loadSolution();
    Solution getSolution();

    void setIteration(unsigned _iteration);
    unsigned getIteration();
    void incIteration();
    
    void setBestValue(int _value_best);
    int getBestValue();

  protected:
    Solution solution;
    Solution solution_best;
    int value_best;
    unsigned iteration;
};

//---------------------------------------------------------------------------
inline void Region::addSolution(int instance_index, int input_index)
{
  solution[instance_index] |= (1 << input_index);
}

//---------------------------------------------------------------------------
inline void Region::removeSolution(int instance_index, int input_index)
{
  solution[instance_index] &= ~(1 << input_index);
}

//---------------------------------------------------------------------------
inline void Region::saveSolution()
{
  memcpy(solution_best, solution, instances->count() * sizeof(unsigned));
}

//---------------------------------------------------------------------------
inline void Region::loadSolution()
{
  memcpy(solution, solution_best, instances->count() * sizeof(unsigned));
}

//---------------------------------------------------------------------------
inline Solution Region::getSolution()
{
  return solution;
}

//---------------------------------------------------------------------------
inline void Region::setBestValue(int _value_best)
{
  value_best = _value_best;
}
//---------------------------------------------------------------------------
inline int Region::getBestValue()
{
  return value_best;
}

//---------------------------------------------------------------------------
inline void Region::setIteration(unsigned _iteration)
{
  iteration = _iteration;
}

//---------------------------------------------------------------------------
inline unsigned Region::getIteration()
{
  return iteration;
}

//---------------------------------------------------------------------------
inline void Region::incIteration()
{
  iteration++;
}

#endif
