#include "sv_set.h"

//---------------------------------------------------------------------------
Set::Set(const char* _name, InstanceArray* _instances, NetArray* _nets, Tag _tag, void* _data)
:Object(_name, _tag, _data)
{
  if (_instances)
    instances = new InstanceArray(_instances->count());
  else
    instances = new InstanceArray(0);
  addInstances(_instances);  

  if (_nets)
    nets = new NetArray(_instances->count());
  else
    nets = new NetArray(0);
  addNets(_nets);  
}

//---------------------------------------------------------------------------
Set::~Set()
{
  delete instances; instances = NULL;
  delete nets; nets = NULL;
}

//---------------------------------------------------------------------------
void Set::addInstance(Instance* instance, int position)
{
  if (position < 0)
    instances->add(instance);
  else
    instances->insert(instance, position);
}

//---------------------------------------------------------------------------
void Set::addInstances(InstanceArray* _instances)
{
  if (_instances)
  for (int instance_index = 0; instance_index < _instances->count(); instance_index++)
    addInstance((*_instances)[instance_index]);
}

//---------------------------------------------------------------------------
void Set::removeInstance(Instance* instance)
{
  instances->remove(instance, false);
}

//---------------------------------------------------------------------------
Instance* Set::findInstance(const char* instance_name) const
{
  Instance* instance_mask = new Instance(instance_name);
  int instance_index = instances->find(instance_mask, 0);
  delete instance_mask;
  if (instance_index >= 0)
    return (*instances)[instance_index];
  else
    return NULL;
}

//---------------------------------------------------------------------------
void Set::addNet(Net* net, int position)
{
  if (position < 0)
    nets->add(net);
  else
    nets->insert(net, position);
}

//---------------------------------------------------------------------------
void Set::addNets(NetArray* _nets)
{
  if (_nets)
  for (int net_index = 0; net_index < _nets->count(); net_index++)
    addNet((*_nets)[net_index]);
}

//---------------------------------------------------------------------------
void Set::removeNet(Net* net)
{
  nets->remove(net, false);
}

//---------------------------------------------------------------------------
Net* Set::findNet(const char* net_name) const
{
  Net* net_mask = new Net(net_name);
  int net_index = nets->find(net_mask, 0);
  delete net_mask;
  if (net_index >= 0)
    return (*nets)[net_index];
  else
    return NULL;
}
